import { VNode, VNodeChild } from './vnode';
import { ReactiveEffect } from '@vue/reactivity';
import { ComponentPublicInstance, ComponentPublicInstanceConstructor } from './componentPublicInstance';
import { ComponentPropsOptions, NormalizedPropsOptions } from './componentProps';
import { Slots, InternalSlots } from './componentSlots';
import { AppContext, AppConfig } from './apiCreateApp';
import { Directive } from './directives';
import { ComponentOptions, ComputedOptions, MethodOptions } from './componentOptions';
import { EmitsOptions, ObjectEmitsOptions, EmitFn } from './componentEmits';
import { SuspenseBoundary } from './components/Suspense';
export declare type Data = Record<string, unknown>;
/**
 * For extending allowed non-declared props on components in TSX
 */
export interface ComponentCustomProps {
}
/**
 * Default allowed non-declared props on component in TSX
 */
export interface AllowedComponentProps {
    class?: unknown;
    style?: unknown;
}
export interface ComponentInternalOptions {
    /**
     * @internal
     */
    __scopeId?: string;
    /**
     * @internal
     */
    __cssModules?: Data;
    /**
     * @internal
     */
    __hmrId?: string;
    /**
     * Compat build only, for bailing out of certain compatibility behavior
     */
    __isBuiltIn?: boolean;
    /**
     * This one should be exposed so that devtools can make use of it
     */
    __file?: string;
}
export interface FunctionalComponent<P = {}, E extends EmitsOptions = {}> extends ComponentInternalOptions {
    (props: P, ctx: Omit<SetupContext<E>, 'expose'>): any;
    props?: ComponentPropsOptions<P>;
    emits?: E | (keyof E)[];
    inheritAttrs?: boolean;
    displayName?: string;
}
export interface ClassComponent {
    new (...args: any[]): ComponentPublicInstance<any, any, any, any, any>;
    __vccOpts: ComponentOptions;
}
/**
 * Concrete component type matches its actual value: it's either an options
 * object, or a function. Use this where the code expects to work with actual
 * values, e.g. checking if its a function or not. This is mostly for internal
 * implementation code.
 */
export declare type ConcreteComponent<Props = {}, RawBindings = any, D = any, C extends ComputedOptions = ComputedOptions, M extends MethodOptions = MethodOptions> = ComponentOptions<Props, RawBindings, D, C, M> | FunctionalComponent<Props, any>;
/**
 * A type used in public APIs where a component type is expected.
 * The constructor type is an artificial type returned by defineComponent().
 */
export declare type Component<Props = any, RawBindings = any, D = any, C extends ComputedOptions = ComputedOptions, M extends MethodOptions = MethodOptions> = ConcreteComponent<Props, RawBindings, D, C, M> | ComponentPublicInstanceConstructor<Props>;
export { ComponentOptions };
declare type LifecycleHook<TFn = Function> = TFn[] | null;
export declare const enum LifecycleHooks {
    BEFORE_CREATE = "bc",
    CREATED = "c",
    BEFORE_MOUNT = "bm",
    MOUNTED = "m",
    BEFORE_UPDATE = "bu",
    UPDATED = "u",
    BEFORE_UNMOUNT = "bum",
    UNMOUNTED = "um",
    DEACTIVATED = "da",
    ACTIVATED = "a",
    RENDER_TRIGGERED = "rtg",
    RENDER_TRACKED = "rtc",
    ERROR_CAPTURED = "ec",
    SERVER_PREFETCH = "sp"
}
export interface SetupContext<E = EmitsOptions> {
    attrs: Data;
    slots: Slots;
    emit: EmitFn<E>;
    expose: (exposed?: Record<string, any>) => void;
}
/**
 * @internal
 */
export declare type InternalRenderFunction = {
    (ctx: ComponentPublicInstance, cache: ComponentInternalInstance['renderCache'], $props: ComponentInternalInstance['props'], $setup: ComponentInternalInstance['setupState'], $data: ComponentInternalInstance['data'], $options: ComponentInternalInstance['ctx']): VNodeChild;
    _rc?: boolean;
    _compatChecked?: boolean;
    _compatWrapped?: boolean;
};
/**
 * We expose a subset of properties on the internal instance as they are
 * useful for advanced external libraries and tools.
 */
export interface ComponentInternalInstance {
    uid: number;
    type: ConcreteComponent;
    parent: ComponentInternalInstance | null;
    root: ComponentInternalInstance;
    appContext: AppContext;
    /**
     * Vnode representing this component in its parent's vdom tree
     */
    vnode: VNode;
    /**
     * The pending new vnode from parent updates
     * @internal
     */
    next: VNode | null;
    /**
     * Root vnode of this component's own vdom tree
     */
    subTree: VNode;
    /**
     * The reactive effect for rendering and patching the component. Callable.
     */
    update: ReactiveEffect;
    /**
     * The render function that returns vdom tree.
     * @internal
     */
    render: InternalRenderFunction | null;
    /**
     * SSR render function
     * @internal
     */
    ssrRender?: Function | null;
    /**
     * Object containing values this component provides for its descendents
     * @internal
     */
    provides: Data;
    /**
     * Tracking reactive effects (e.g. watchers) associated with this component
     * so that they can be automatically stopped on component unmount
     * @internal
     */
    effects: ReactiveEffect[] | null;
    /**
     * cache for proxy access type to avoid hasOwnProperty calls
     * @internal
     */
    accessCache: Data | null;
    /**
     * cache for render function values that rely on _ctx but won't need updates
     * after initialized (e.g. inline handlers)
     * @internal
     */
    renderCache: (Function | VNode)[];
    /**
     * Resolved component registry, only for components with mixins or extends
     * @internal
     */
    components: Record<string, ConcreteComponent> | null;
    /**
     * Resolved directive registry, only for components with mixins or extends
     * @internal
     */
    directives: Record<string, Directive> | null;
    /**
     * Resolved filters registry, v2 compat only
     * @internal
     */
    filters?: Record<string, Function>;
    /**
     * resolved props options
     * @internal
     */
    propsOptions: NormalizedPropsOptions;
    /**
     * resolved emits options
     * @internal
     */
    emitsOptions: ObjectEmitsOptions | null;
    /**
     * resolved inheritAttrs options
     * @internal
     */
    inheritAttrs?: boolean;
    proxy: ComponentPublicInstance | null;
    exposed: Record<string, any> | null;
    exposeProxy: Record<string, any> | null;
    /**
     * alternative proxy used only for runtime-compiled render functions using
     * `with` block
     * @internal
     */
    withProxy: ComponentPublicInstance | null;
    /**
     * This is the target for the public instance proxy. It also holds properties
     * injected by user options (computed, methods etc.) and user-attached
     * custom properties (via `this.x = ...`)
     * @internal
     */
    ctx: Data;
    data: Data;
    props: Data;
    attrs: Data;
    slots: InternalSlots;
    refs: Data;
    emit: EmitFn;
    /**
     * used for keeping track of .once event handlers on components
     * @internal
     */
    emitted: Record<string, boolean> | null;
    /**
     * used for caching the value returned from props default factory functions to
     * avoid unnecessary watcher trigger
     * @internal
     */
    propsDefaults: Data;
    /**
     * setup related
     * @internal
     */
    setupState: Data;
    /**
     * devtools access to additional info
     * @internal
     */
    devtoolsRawSetupState?: any;
    /**
     * @internal
     */
    setupContext: SetupContext | null;
    /**
     * suspense related
     * @internal
     */
    suspense: SuspenseBoundary | null;
    /**
     * suspense pending batch id
     * @internal
     */
    suspenseId: number;
    /**
     * @internal
     */
    asyncDep: Promise<any> | null;
    /**
     * @internal
     */
    asyncResolved: boolean;
    isMounted: boolean;
    isUnmounted: boolean;
    isDeactivated: boolean;
    /**
     * @internal
     */
    [LifecycleHooks.BEFORE_CREATE]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.CREATED]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.BEFORE_MOUNT]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.MOUNTED]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.BEFORE_UPDATE]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.UPDATED]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.BEFORE_UNMOUNT]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.UNMOUNTED]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.RENDER_TRACKED]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.RENDER_TRIGGERED]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.ACTIVATED]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.DEACTIVATED]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.ERROR_CAPTURED]: LifecycleHook;
    /**
     * @internal
     */
    [LifecycleHooks.SERVER_PREFETCH]: LifecycleHook<() => Promise<unknown>>;
}
export declare function createComponentInstance(vnode: VNode, parent: ComponentInternalInstance | null, suspense: SuspenseBoundary | null): ComponentInternalInstance;
export declare let currentInstance: ComponentInternalInstance | null;
export declare const getCurrentInstance: () => ComponentInternalInstance | null;
export declare const setCurrentInstance: (instance: ComponentInternalInstance | null) => void;
export declare function validateComponentName(name: string, config: AppConfig): void;
export declare function isStatefulComponent(instance: ComponentInternalInstance): number;
export declare let isInSSRComponentSetup: boolean;
export declare function setupComponent(instance: ComponentInternalInstance, isSSR?: boolean): Promise<void> | undefined;
export declare function handleSetupResult(instance: ComponentInternalInstance, setupResult: unknown, isSSR: boolean): void;
export declare const isRuntimeOnly: () => boolean;
/**
 * For runtime-dom to register the compiler.
 * Note the exported method uses any to avoid d.ts relying on the compiler types.
 */
export declare function registerRuntimeCompiler(_compile: any): void;
export declare function finishComponentSetup(instance: ComponentInternalInstance, isSSR: boolean, skipOptions?: boolean): void;
export declare function createSetupContext(instance: ComponentInternalInstance): SetupContext;
export declare function getExposeProxy(instance: ComponentInternalInstance): Record<string, any> | undefined;
export declare function recordInstanceBoundEffect(effect: ReactiveEffect, instance?: ComponentInternalInstance | null): void;
export declare function getComponentName(Component: ConcreteComponent): string | undefined;
export declare function formatComponentName(instance: ComponentInternalInstance | null, Component: ConcreteComponent, isRoot?: boolean): string;
export declare function isClassComponent(value: unknown): value is ClassComponent;
//# sourceMappingURL=component.d.ts.map